# -*- coding:utf-8 -*-
# author：huawei
from python2sky.proto.common.trace_common_pb2 import CrossProcess
from python2sky.proto.language_agent_v2.trace_pb2 import SegmentReference
from python2sky.util.common import null_value, build_unique_id
from python2sky.util.string_util import is_empty
from python2sky.util.uuid_util import global_id_to_string, string_to_global_id
from python2sky.util.base64_util import encode, decode


def encode_compressed_field(id, text):
    if id and id != 0:
        return encode(str(id))
    return encode("#" + text)


def decode_field(text):
    text = decode(text)
    if text and text.startswith("#"):
        return text[1:], 0
    return None, int(text)


class ContextCarrier:
    def __init__(self):
        self.trace_segment_id = None
        self.span_id = -1
        self.parent_service_instance_id = 0
        self.entry_service_instance_id = 0
        self.peer = None
        self.peer_id = None
        self.entry_endpoint_name = None
        self.parent_endpoint_name = None
        self.trace_id = None
        self.sample = None
        self.parent_endpoint_id = 0
        self.network_address_id = None
        self.entry_endpoint_id = None
        self.type = None

    def deserialize(self, text):
        parts = text.split("-", 9)
        self.sample = parts[0]
        self.trace_id = string_to_global_id(decode(parts[1]))
        self.trace_segment_id = string_to_global_id(decode(parts[2]))
        self.span_id = int(parts[3])
        self.parent_service_instance_id = int(parts[4])
        self.entry_service_instance_id = int(parts[5])
        self.peer, self.peer_id = decode_field(parts[6])
        self.entry_endpoint_name, self.entry_endpoint_id = decode_field(parts[7])
        self.parent_endpoint_name, self.parent_endpoint_id = decode_field(parts[8])

    def serialize(self):
        if self.trace_id is None:
            return None
        return "-".join(["1",
                         encode(global_id_to_string(self.trace_id)),
                         encode(global_id_to_string(self.trace_segment_id)),
                         str(self.span_id),
                         str(self.parent_service_instance_id),
                         str(self.entry_service_instance_id),
                         encode_compressed_field(self.network_address_id, self.peer),
                         encode_compressed_field(self.entry_endpoint_id, self.entry_endpoint_name),
                         encode_compressed_field(self.parent_endpoint_id, self.parent_endpoint_name)
                         ])

    def transform(self):
        segment_reference = SegmentReference()

        if self.type == CrossProcess:
            segment_reference.refType = self.type
            if null_value(self.peer_id):
                segment_reference.networkAddress = self.peer
            else:
                segment_reference.networkAddressId = self.peer_id
        else:
            segment_reference.refType = self.type

        segment_reference.parentServiceInstanceId = self.parent_service_instance_id
        segment_reference.entryServiceInstanceId = self.entry_service_instance_id
        segment_reference.parentTraceSegmentId.CopyFrom(build_unique_id(self.trace_segment_id))
        segment_reference.parentSpanId = self.span_id

        if null_value(self.entry_endpoint_id):
            if not is_empty(self.entry_endpoint_name):
                segment_reference.entryEndpoint = self.entry_endpoint_name
        else:
            segment_reference.entryEndpointId = self.entry_endpoint_id

        if null_value(self.parent_endpoint_id):
            if not is_empty(self.parent_endpoint_name):
                segment_reference.parentEndpoint = self.parent_endpoint_name
        else:
            segment_reference.parentEndpointId = self.parent_endpoint_id

        return segment_reference










# -*- coding:utf-8 -*-
# author：huawei
from python2sky.context.span import Span


class NoopSpan(Span):

    def is_entry(self):
        return False

    def is_exit(self):
        return False

    def tag(self, key, value):
        return self

    def start(self):
        return self

    def end(self):
        return self

    def ref(self, context_carrier):
        pass

    def log(self, ex):
        return self

    def transform(self):
        pass


# -*- coding:utf-8 -*-
# author：huawei
import os
import platform
import socket


def get_all_ipv4():
    ipv4 = set()
    addrs = socket.getaddrinfo(socket.gethostname(), None)
    for item in addrs:
        if ':' not in item[4][0] and "127.0.0.1" != item[4][0]:
            ipv4.add(item[4][0])
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(("8.8.8.8", 80))
    ipv4.add(s.getsockname()[0])
    s.close()
    return ipv4


def get_host_name():
    return socket.gethostname()


def get_process_no():
    return os.getpid()


def get_os_name():
    return platform.system()


if __name__ == "__main__":
    print(get_all_ipv4())
    print(get_host_name())
    print(get_process_no())
    print(get_os_name())



# -*- coding:utf-8 -*-
# author：huawei


BACKEND_SERVICE = "127.0.0.1:11800"

SERVICE_ID = 0

SERVICE_NAME = "python-test"

SERVICE_INSTANCE_ID = 0

IGNORE_SUFFIX = ["ico", "css", "jpg", "js"]

OPERATION_NAME_THRESHOLD = 500

AGENT_VERSION = ""

REGISTER_INTERVAL = 10

SKYWALKING_HERADER_V2 = "sw6"



# -*- coding:utf-8 -*-
# author：huawei
import traceback
from abc import ABC

from python2sky.proto.common.trace_common_pb2 import Entry, Exit
from python2sky.util.common import build_key_value, null_value
from python2sky.proto.language_agent_v2.trace_pb2 import Log, SpanObjectV2
from python2sky.util.date_util import current_milli_time
from python2sky.util.string_util import is_empty


class Span(ABC):
    def __init__(self):
        self.span_id = -1
        self.parent_span_id = None
        self.tags = None
        self.operation_name = None
        self.operation_id = None
        self.logs = None
        self.start_time = None
        self.end_time = None
        self.error_occurred = False
        self.component_id = 0
        self.component_name = ""
        self.refs = None
        self.type = None
        self.layer = None
        self.peer = None
        self.peer_id = None
        self.error_occurred = False

    def is_entry(self):
        return self.type == Entry

    def is_exit(self):
        return self.type == Exit

    def tag(self, key, value):
        if not self.tags:
            self.tags = []
        self.tags.append(build_key_value(key, value))
        return self

    def start(self):
        self.start_time = current_milli_time()
        return self

    def end(self):
        self.end_time = current_milli_time()
        return self

    def ref(self, context_carrier):
        if not self.refs:
            self.refs = []
        if context_carrier not in self.refs:
            self.refs.append(context_carrier)

    def log(self, ex):
        if not self.logs:
            self.logs = []
        self.error_occurred = True
        log = Log()
        log.time = current_milli_time()
        log.data.append(build_key_value("event", "error"))
        log.data.append(build_key_value("error.kind", type(ex)))
        log.data.append(build_key_value("message", str(ex)))
        log.data.append(build_key_value("stack", traceback.format_exc()))
        self.logs.append(log)
        return self

    def transform(self):
        span_object_v2 = SpanObjectV2()
        span_object_v2.spanId = self.span_id
        span_object_v2.parentSpanId = self.parent_span_id
        span_object_v2.startTime = self.start_time
        span_object_v2.endTime = self.end_time
        if not null_value(self.operation_id):
            span_object_v2.operationNameId = self.operation_id
        else:
            span_object_v2.operationName = self.operation_name

        span_object_v2.spanType = self.type

        if self.layer is not None:
            span_object_v2.spanLayer = self.layer

        if not null_value(self.component_id):
            span_object_v2.componentId = self.component_id
        else:
            if not is_empty(self.component_name):
                span_object_v2.component = self.component_name

        span_object_v2.isError = self.error_occurred

        if self.tags:
            for tag in self.tags:
                span_object_v2.tags.append(tag)

        if self.logs:
            for log in self.logs:
                span_object_v2.logs.append(log)

        if self.refs:
            for ref in self.refs:
                span_object_v2.refs.append(ref.transform())

        if not null_value(self.peer_id):
            span_object_v2.peerId = self.peer_id
        else:
            if not is_empty(self.peer):
                span_object_v2.peer = self.peer

        return span_object_v2



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: common/CLR.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from python2sky.proto.common import common_pb2 as common_dot_common__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
  name='common/CLR.proto',
  package='',
  syntax='proto3',
  serialized_options=b'\n0org.apache.skywalking.apm.network.language.agentP\001\252\002\032SkyWalking.NetworkProtocol',
  serialized_pb=b'\n\x10\x63ommon/CLR.proto\x1a\x13\x63ommon/common.proto\"\\\n\tCLRMetric\x12\x0c\n\x04time\x18\x01 \x01(\x03\x12\x11\n\x03\x63pu\x18\x02 \x01(\x0b\x32\x04.CPU\x12\x12\n\x02gc\x18\x03 \x01(\x0b\x32\x06.ClrGC\x12\x1a\n\x06thread\x18\x04 \x01(\x0b\x32\n.ClrThread\"i\n\x05\x43lrGC\x12\x18\n\x10Gen0CollectCount\x18\x01 \x01(\x03\x12\x18\n\x10Gen1CollectCount\x18\x02 \x01(\x03\x12\x18\n\x10Gen2CollectCount\x18\x03 \x01(\x03\x12\x12\n\nHeapMemory\x18\x04 \x01(\x03\"\x8f\x01\n\tClrThread\x12&\n\x1e\x41vailableCompletionPortThreads\x18\x01 \x01(\x05\x12\x1e\n\x16\x41vailableWorkerThreads\x18\x02 \x01(\x05\x12 \n\x18MaxCompletionPortThreads\x18\x03 \x01(\x05\x12\x18\n\x10MaxWorkerThreads\x18\x04 \x01(\x05\x42Q\n0org.apache.skywalking.apm.network.language.agentP\x01\xaa\x02\x1aSkyWalking.NetworkProtocolb\x06proto3'
  ,
  dependencies=[common_dot_common__pb2.DESCRIPTOR,])




_CLRMETRIC = _descriptor.Descriptor(
  name='CLRMetric',
  full_name='CLRMetric',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='time', full_name='CLRMetric.time', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cpu', full_name='CLRMetric.cpu', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gc', full_name='CLRMetric.gc', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thread', full_name='CLRMetric.thread', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=41,
  serialized_end=133,
)


_CLRGC = _descriptor.Descriptor(
  name='ClrGC',
  full_name='ClrGC',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='Gen0CollectCount', full_name='ClrGC.Gen0CollectCount', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Gen1CollectCount', full_name='ClrGC.Gen1CollectCount', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Gen2CollectCount', full_name='ClrGC.Gen2CollectCount', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='HeapMemory', full_name='ClrGC.HeapMemory', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=135,
  serialized_end=240,
)


_CLRTHREAD = _descriptor.Descriptor(
  name='ClrThread',
  full_name='ClrThread',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='AvailableCompletionPortThreads', full_name='ClrThread.AvailableCompletionPortThreads', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='AvailableWorkerThreads', full_name='ClrThread.AvailableWorkerThreads', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='MaxCompletionPortThreads', full_name='ClrThread.MaxCompletionPortThreads', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='MaxWorkerThreads', full_name='ClrThread.MaxWorkerThreads', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=243,
  serialized_end=386,
)

_CLRMETRIC.fields_by_name['cpu'].message_type = common_dot_common__pb2._CPU
_CLRMETRIC.fields_by_name['gc'].message_type = _CLRGC
_CLRMETRIC.fields_by_name['thread'].message_type = _CLRTHREAD
DESCRIPTOR.message_types_by_name['CLRMetric'] = _CLRMETRIC
DESCRIPTOR.message_types_by_name['ClrGC'] = _CLRGC
DESCRIPTOR.message_types_by_name['ClrThread'] = _CLRTHREAD
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CLRMetric = _reflection.GeneratedProtocolMessageType('CLRMetric', (_message.Message,), {
  'DESCRIPTOR' : _CLRMETRIC,
  '__module__' : 'common.CLR_pb2'
  # @@protoc_insertion_point(class_scope:CLRMetric)
  })
_sym_db.RegisterMessage(CLRMetric)

ClrGC = _reflection.GeneratedProtocolMessageType('ClrGC', (_message.Message,), {
  'DESCRIPTOR' : _CLRGC,
  '__module__' : 'common.CLR_pb2'
  # @@protoc_insertion_point(class_scope:ClrGC)
  })
_sym_db.RegisterMessage(ClrGC)

ClrThread = _reflection.GeneratedProtocolMessageType('ClrThread', (_message.Message,), {
  'DESCRIPTOR' : _CLRTHREAD,
  '__module__' : 'common.CLR_pb2'
  # @@protoc_insertion_point(class_scope:ClrThread)
  })
_sym_db.RegisterMessage(ClrThread)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: language_agent_v2/CLRMetric.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from python2sky.proto.common import common_pb2 as common_dot_common__pb2
from python2sky.proto.common import CLR_pb2 as common_dot_CLR__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='language_agent_v2/CLRMetric.proto',
  package='',
  syntax='proto3',
  serialized_options=b'\n3org.apache.skywalking.apm.network.language.agent.v2P\001\252\002\032SkyWalking.NetworkProtocol',
  serialized_pb=b'\n!language_agent_v2/CLRMetric.proto\x1a\x13\x63ommon/common.proto\x1a\x10\x63ommon/CLR.proto\"M\n\x13\x43LRMetricCollection\x12\x1b\n\x07metrics\x18\x01 \x03(\x0b\x32\n.CLRMetric\x12\x19\n\x11serviceInstanceId\x18\x02 \x01(\x05\x32\x46\n\x16\x43LRMetricReportService\x12,\n\x07\x63ollect\x12\x14.CLRMetricCollection\x1a\t.Commands\"\x00\x42T\n3org.apache.skywalking.apm.network.language.agent.v2P\x01\xaa\x02\x1aSkyWalking.NetworkProtocolb\x06proto3'
  ,
  dependencies=[common_dot_common__pb2.DESCRIPTOR,common_dot_CLR__pb2.DESCRIPTOR,])




_CLRMETRICCOLLECTION = _descriptor.Descriptor(
  name='CLRMetricCollection',
  full_name='CLRMetricCollection',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='metrics', full_name='CLRMetricCollection.metrics', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='serviceInstanceId', full_name='CLRMetricCollection.serviceInstanceId', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=76,
  serialized_end=153,
)

_CLRMETRICCOLLECTION.fields_by_name['metrics'].message_type = common_dot_CLR__pb2._CLRMETRIC
DESCRIPTOR.message_types_by_name['CLRMetricCollection'] = _CLRMETRICCOLLECTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CLRMetricCollection = _reflection.GeneratedProtocolMessageType('CLRMetricCollection', (_message.Message,), {
  'DESCRIPTOR' : _CLRMETRICCOLLECTION,
  '__module__' : 'language_agent_v2.CLRMetric_pb2'
  # @@protoc_insertion_point(class_scope:CLRMetricCollection)
  })
_sym_db.RegisterMessage(CLRMetricCollection)


DESCRIPTOR._options = None

_CLRMETRICREPORTSERVICE = _descriptor.ServiceDescriptor(
  name='CLRMetricReportService',
  full_name='CLRMetricReportService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=155,
  serialized_end=225,
  methods=[
  _descriptor.MethodDescriptor(
    name='collect',
    full_name='CLRMetricReportService.collect',
    index=0,
    containing_service=None,
    input_type=_CLRMETRICCOLLECTION,
    output_type=common_dot_common__pb2._COMMANDS,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_CLRMETRICREPORTSERVICE)

DESCRIPTOR.services_by_name['CLRMetricReportService'] = _CLRMETRICREPORTSERVICE

# @@protoc_insertion_point(module_scope)



# -*- coding:utf-8 -*-
# author：huawei
import threading

from python2sky.context.context_carrier import ContextCarrier
from python2sky.context.noop_span import NoopSpan
from python2sky.context.span import Span
from python2sky.context.trace_segment import TraceSegment
from python2sky.exception.exceptions import SkywalkingException
from python2sky.proto.common.trace_common_pb2 import Entry, Local, Exit, RefType, CrossProcess, CrossThread
from python2sky.util.common import null_value
from python2sky.util.string_util import is_empty
from python2sky.util.uuid_util import global_id_to_string

INEXISTENCE = -1

from abc import ABC


class AbstractTracerContext(ABC):
    def inject(self, context_carrier):
        pass

    def extract(self, context_carrier):
        pass

    def capture(self):
        pass

    def continued(self, snapshot):
        pass

    def get_readable_global_trace_id(self):
        pass

    def create_entry_span(self, operation_name):
        pass

    def create_local_span(self, operation_name):
        pass

    def create_exit_span(self, operation_name, remote_peer):
        pass

    def active_span(self):
        pass

    def stop_span(self, span):
        pass

    def await_finish_async(self):
        pass

    def async_stop(self):
        pass


class TracingContext(AbstractTracerContext):
    def __init__(self):
        self.segment = TraceSegment()
        self.spans = []
        self.span_id_generator = 0

    def inject(self, context_carrier):
        span = self.active_span()
        if not span.is_exit():
            raise SkywalkingException("Inject can be done only in Exit Span")
        context_carrier.peer = span.peer
        context_carrier.trace_segment_id = self.segment.trace_segment_id
        context_carrier.span_id = span.span_id
        context_carrier.parent_service_instance_id = self.segment.application_instance_id

        first_span = self.first()
        operation_id = INEXISTENCE
        first_span_operation_name = first_span.operation_name

        if self.segment.refs and len(self.segment.refs) > 0:
            ref = self.segment.refs[0]
            operation_id = ref.entry_endpoint_id
            operation_name = ref.entry_endpoint_name
            entry_service_instance_id = ref.entry_service_instance_id
        else:
            if first_span.is_entry():
                operation_id = first_span.operation_id
                operation_name = first_span_operation_name
            entry_service_instance_id = self.segment.application_instance_id

        context_carrier.entry_service_instance_id = entry_service_instance_id

        if null_value(operation_id):
            if not is_empty(operation_name):
                context_carrier.entry_endpoint_name = operation_name
        else:
            context_carrier.entry_endpoint_id = operation_id

        parent_operation_id = first_span.operation_id
        if null_value(parent_operation_id):
            if first_span.is_entry() and not is_empty(first_span_operation_name):
                context_carrier.parent_endpoint_name = first_span_operation_name
            else:
                context_carrier.parent_endpoint_id = INEXISTENCE
        else:
            context_carrier.parent_endpoint_id = operation_id

        context_carrier.trace_id = self.segment.trace_ids[0]

    def extract(self, context_carrier):
        context_carrier.type = CrossProcess
        self.segment.ref(context_carrier)
        self.segment.related_global_traces(context_carrier.trace_id)
        span = self.active_span()
        if span.is_entry():
            span.ref(context_carrier)

    def capture(self):
        context_carrier = ContextCarrier()
        context_carrier.trace_segment_id = self.segment.trace_segment_id
        context_carrier.span_id = self.active_span().span_id
        context_carrier.trace_id = self.segment.get_related_global_traces()[0]
        entry_operation_name = None
        first_span = self.first()
        first_span_operation_name = first_span.operation_name

        if self.segment.refs and len(self.segment.refs) > 0:
            ref = self.segment.refs[0]
            entry_operation_id = ref.entry_endpoint_id
            entry_operation_name = ref.entry_endpoint_name
            entry_service_instance_id = ref.entry_service_instance_id
        else:
            if first_span.is_entry():
                entry_operation_id = first_span.operation_id
                entry_operation_name = first_span_operation_name
            else:
                entry_operation_id = INEXISTENCE

            entry_service_instance_id = self.segment.application_instance_id

        context_carrier.entry_service_instance_id = entry_service_instance_id

        if null_value(entry_operation_id):
            if not is_empty(entry_operation_name):
                context_carrier.entry_endpoint_name = entry_operation_name
        else:
            context_carrier.entry_endpoint_id = entry_operation_id

        parent_operation_id = first_span.operation_id
        if null_value(parent_operation_id):
            if first_span.is_entry() and not is_empty(first_span_operation_name):
                context_carrier.parent_endpoint_name = first_span_operation_name
            else:
                context_carrier.parent_endpoint_id = INEXISTENCE
        else:
            context_carrier.parent_endpoint_id = parent_operation_id

        context_carrier.parent_service_instance_id = self.segment.application_instance_id

        return context_carrier

    def continued(self, context_carrier):
        context_carrier.type = CrossThread
        self.segment.ref(context_carrier)
        self.active_span().ref(context_carrier)
        self.segment.related_global_traces(context_carrier.trace_id)

    def get_readable_global_trace_id(self):
        return global_id_to_string(self.segment.get_related_global_traces()[0])

    def create_entry_span(self, operation_name):
        if self.is_limit_mechanism_working():
            noopSpan = NoopSpan()
            self.spans.append(noopSpan)
            return noopSpan

        parent_span = self.peek()
        parent_span_id = parent_span.span_id if parent_span else -1

        if parent_span and parent_span.is_entry():
            parent_span.operation_name = operation_name
            entry_span = parent_span
            return entry_span.start()
        else:
            entry_span = self.__create_span(operation_name, Entry, parent_span_id)

        return self.push(entry_span)

    def create_local_span(self, operation_name):
        if self.is_limit_mechanism_working():
            noopSpan = NoopSpan()
            self.spans.append(noopSpan)
            return noopSpan

        parent_span = self.peek()
        parent_span_id = parent_span.span_id if parent_span else -1
        local_span = self.__create_span(operation_name, Local, parent_span_id)

        return self.push(local_span)

    def create_exit_span(self, operation_name, remote_peer):
        if self.is_limit_mechanism_working():
            noopSpan = NoopSpan()
            self.spans.append(noopSpan)
            return noopSpan

        parent_span = self.peek()
        parent_span_id = parent_span.span_id if parent_span else -1

        if parent_span and parent_span.is_exit():
            parent_span.operation_name = operation_name
            exit_span = parent_span
            return exit_span.start()
        else:
            exit_span = self.__create_span(operation_name, Exit, parent_span_id)
            exit_span.peer = remote_peer

        return self.push(exit_span)

    def __create_span(self, operation_name, span_type, parent_span_id):
        span = Span()
        span.span_id = self.span_id_generator
        self.span_id_generator += 1
        span.parent_span_id = parent_span_id
        span.operation_name = operation_name
        span.type = span_type

        return span.start()

    def push(self, span):
        self.spans.append(span)
        return span

    def active_span(self):
        span = self.peek()
        if span is None:
            raise SkywalkingException("No active span.")
        return span

    def peek(self):
        if self.spans:
            return self.spans[len(self.spans) - 1]
        return None

    def first(self):
        if self.spans:
            return self.spans[0]
        return None

    def is_limit_mechanism_working(self):
        return False

    def stop_span(self, span):
        last_span = self.active_span()
        if span == last_span:
            if type(last_span) != NoopSpan:
                self.segment.archive(span)
            self.spans.pop()

        else:
            raise SkywalkingException("Stopping the unexpected span = " + str(span.type))
        self.finish()
        return len(self.spans) == 0

    def finish(self):
        if len(self.spans) == 0:
            ListenerManager.notify_finish(self.segment)





class ListenerManager:
    __LISTENERS = []
    __lock = threading.Lock()

    @classmethod
    def notify_finish(cls, trace_segment):
        for listener in ListenerManager.__LISTENERS:
            listener.after_finished(trace_segment)

    @classmethod
    def add(cls, listener):
        with ListenerManager.__lock:
            ListenerManager.__LISTENERS.append(listener)

    @classmethod
    def remove(cls, listener):
        with ListenerManager.__lock:
            ListenerManager.__LISTENERS.remove(listener)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: register/InstancePing.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from python2sky.proto.common import common_pb2 as common_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='register/InstancePing.proto',
  package='',
  syntax='proto3',
  serialized_options=b'\n-org.apache.skywalking.apm.network.register.v2P\001\252\002\032SkyWalking.NetworkProtocol',
  serialized_pb=b'\n\x1bregister/InstancePing.proto\x1a\x13\x63ommon/common.proto\"^\n\x16ServiceInstancePingPkg\x12\x19\n\x11serviceInstanceId\x18\x01 \x01(\x05\x12\x0c\n\x04time\x18\x02 \x01(\x03\x12\x1b\n\x13serviceInstanceUUID\x18\x03 \x01(\t2E\n\x13ServiceInstancePing\x12.\n\x06\x64oPing\x12\x17.ServiceInstancePingPkg\x1a\t.Commands\"\x00\x42N\n-org.apache.skywalking.apm.network.register.v2P\x01\xaa\x02\x1aSkyWalking.NetworkProtocolb\x06proto3'
  ,
  dependencies=[common_dot_common__pb2.DESCRIPTOR,])




_SERVICEINSTANCEPINGPKG = _descriptor.Descriptor(
  name='ServiceInstancePingPkg',
  full_name='ServiceInstancePingPkg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='serviceInstanceId', full_name='ServiceInstancePingPkg.serviceInstanceId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='time', full_name='ServiceInstancePingPkg.time', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='serviceInstanceUUID', full_name='ServiceInstancePingPkg.serviceInstanceUUID', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=52,
  serialized_end=146,
)

DESCRIPTOR.message_types_by_name['ServiceInstancePingPkg'] = _SERVICEINSTANCEPINGPKG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ServiceInstancePingPkg = _reflection.GeneratedProtocolMessageType('ServiceInstancePingPkg', (_message.Message,), {
  'DESCRIPTOR' : _SERVICEINSTANCEPINGPKG,
  '__module__' : 'register.InstancePing_pb2'
  # @@protoc_insertion_point(class_scope:ServiceInstancePingPkg)
  })
_sym_db.RegisterMessage(ServiceInstancePingPkg)


DESCRIPTOR._options = None

_SERVICEINSTANCEPING = _descriptor.ServiceDescriptor(
  name='ServiceInstancePing',
  full_name='ServiceInstancePing',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=148,
  serialized_end=217,
  methods=[
  _descriptor.MethodDescriptor(
    name='doPing',
    full_name='ServiceInstancePing.doPing',
    index=0,
    containing_service=None,
    input_type=_SERVICEINSTANCEPINGPKG,
    output_type=common_dot_common__pb2._COMMANDS,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_SERVICEINSTANCEPING)

DESCRIPTOR.services_by_name['ServiceInstancePing'] = _SERVICEINSTANCEPING

# @@protoc_insertion_point(module_scope)



# -*- coding:utf-8 -*-
# author：huawei
import threading

from python2sky.context.context_carrier import ContextCarrier
from python2sky.context.noop_span import NoopSpan
from python2sky.context.trace_context import AbstractTracerContext


class IgnoredTracerContext(AbstractTracerContext):
    __noop_span = NoopSpan()

    def __init__(self):
        self.stack_depth = 0

    def inject(self, context_carrier):
        pass

    def extract(self, context_carrier):
        pass

    def capture(self):
        return ContextCarrier()

    def continued(self, snapshot):
        pass

    def get_readable_global_trace_id(self):
        return "[Ignored Trace]"

    def create_entry_span(self, operation_name):
        self.stack_depth += 1
        return IgnoredTracerContext.__noop_span

    def create_local_span(self, operation_name):
        self.stack_depth += 1
        return IgnoredTracerContext.__noop_span

    def create_exit_span(self, operation_name, remote_peer):
        self.stack_depth += 1
        return IgnoredTracerContext.__noop_span

    def active_span(self):
        return IgnoredTracerContext.__noop_span

    def stop_span(self, span):
        self.stack_depth -= 1
        if self.stack_depth == 0:
            IgnoredListenerManager.notify_finish(self)

    def await_finish_async(self):
        pass

    def async_stop(self):
        pass


class IgnoredListenerManager:
    __LISTENERS = []
    __lock = threading.Lock()

    @classmethod
    def notify_finish(cls, ignoredTracerContext):
        for listener in IgnoredListenerManager.__LISTENERS:
            listener.after_finished(ignoredTracerContext)

    @classmethod
    def add(cls, listener):
        with IgnoredListenerManager.__lock:
            IgnoredListenerManager.__LISTENERS.append(listener)

    @classmethod
    def remove(cls, listener):
        with IgnoredListenerManager.__lock:
            IgnoredListenerManager.__LISTENERS.remove(listener)



# -*- coding:utf-8 -*-
# author：huawei
import os
import random
import threading
import time
import uuid

from python2sky.util.date_util import current_milli_time


def get_uuid():
    uid = uuid.uuid1()
    uid = str(uid).replace("-", "")
    return uid


lock = threading.Lock()


def global_id_generator(instance_id):
    with lock:
        return [str(instance_id), str(current_milli_time()), random_id()]


def random_id():
    return int(round(time.time() * 10000000)) + random.randrange(11111, 99999)


def global_id_to_string(ids):
    return str(ids[0]) + "." + str(ids[1]) + "." + str(ids[2])


def string_to_global_id(id_str):
    return id_str.split(".")


if __name__ == '__main__':
    print(random_id())
    print(get_uuid())
    ids = global_id_generator(1)
    print(ids)
    ids_str = global_id_to_string(ids)

    print(ids_str)
    print(string_to_global_id(ids_str))



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from python2sky.proto.common import common_pb2 as common_dot_common__pb2
from python2sky.proto.register import Register_pb2 as register_dot_Register__pb2


class RegisterStub(object):
    """register service for ApplicationCode, this service is called when service starts.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.doServiceRegister = channel.unary_unary(
                '/Register/doServiceRegister',
                request_serializer=register_dot_Register__pb2.Services.SerializeToString,
                response_deserializer=register_dot_Register__pb2.ServiceRegisterMapping.FromString,
                )
        self.doServiceInstanceRegister = channel.unary_unary(
                '/Register/doServiceInstanceRegister',
                request_serializer=register_dot_Register__pb2.ServiceInstances.SerializeToString,
                response_deserializer=register_dot_Register__pb2.ServiceInstanceRegisterMapping.FromString,
                )
        self.doEndpointRegister = channel.unary_unary(
                '/Register/doEndpointRegister',
                request_serializer=register_dot_Register__pb2.Endpoints.SerializeToString,
                response_deserializer=register_dot_Register__pb2.EndpointMapping.FromString,
                )
        self.doNetworkAddressRegister = channel.unary_unary(
                '/Register/doNetworkAddressRegister',
                request_serializer=register_dot_Register__pb2.NetAddresses.SerializeToString,
                response_deserializer=register_dot_Register__pb2.NetAddressMapping.FromString,
                )
        self.doServiceAndNetworkAddressMappingRegister = channel.unary_unary(
                '/Register/doServiceAndNetworkAddressMappingRegister',
                request_serializer=register_dot_Register__pb2.ServiceAndNetworkAddressMappings.SerializeToString,
                response_deserializer=common_dot_common__pb2.Commands.FromString,
                )


class RegisterServicer(object):
    """register service for ApplicationCode, this service is called when service starts.
    """

    def doServiceRegister(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def doServiceInstanceRegister(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def doEndpointRegister(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def doNetworkAddressRegister(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def doServiceAndNetworkAddressMappingRegister(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_RegisterServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'doServiceRegister': grpc.unary_unary_rpc_method_handler(
                    servicer.doServiceRegister,
                    request_deserializer=register_dot_Register__pb2.Services.FromString,
                    response_serializer=register_dot_Register__pb2.ServiceRegisterMapping.SerializeToString,
            ),
            'doServiceInstanceRegister': grpc.unary_unary_rpc_method_handler(
                    servicer.doServiceInstanceRegister,
                    request_deserializer=register_dot_Register__pb2.ServiceInstances.FromString,
                    response_serializer=register_dot_Register__pb2.ServiceInstanceRegisterMapping.SerializeToString,
            ),
            'doEndpointRegister': grpc.unary_unary_rpc_method_handler(
                    servicer.doEndpointRegister,
                    request_deserializer=register_dot_Register__pb2.Endpoints.FromString,
                    response_serializer=register_dot_Register__pb2.EndpointMapping.SerializeToString,
            ),
            'doNetworkAddressRegister': grpc.unary_unary_rpc_method_handler(
                    servicer.doNetworkAddressRegister,
                    request_deserializer=register_dot_Register__pb2.NetAddresses.FromString,
                    response_serializer=register_dot_Register__pb2.NetAddressMapping.SerializeToString,
            ),
            'doServiceAndNetworkAddressMappingRegister': grpc.unary_unary_rpc_method_handler(
                    servicer.doServiceAndNetworkAddressMappingRegister,
                    request_deserializer=register_dot_Register__pb2.ServiceAndNetworkAddressMappings.FromString,
                    response_serializer=common_dot_common__pb2.Commands.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'Register', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Register(object):
    """register service for ApplicationCode, this service is called when service starts.
    """

    @staticmethod
    def doServiceRegister(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Register/doServiceRegister',
            register_dot_Register__pb2.Services.SerializeToString,
            register_dot_Register__pb2.ServiceRegisterMapping.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def doServiceInstanceRegister(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Register/doServiceInstanceRegister',
            register_dot_Register__pb2.ServiceInstances.SerializeToString,
            register_dot_Register__pb2.ServiceInstanceRegisterMapping.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def doEndpointRegister(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Register/doEndpointRegister',
            register_dot_Register__pb2.Endpoints.SerializeToString,
            register_dot_Register__pb2.EndpointMapping.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def doNetworkAddressRegister(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Register/doNetworkAddressRegister',
            register_dot_Register__pb2.NetAddresses.SerializeToString,
            register_dot_Register__pb2.NetAddressMapping.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def doServiceAndNetworkAddressMappingRegister(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Register/doServiceAndNetworkAddressMappingRegister',
            register_dot_Register__pb2.ServiceAndNetworkAddressMappings.SerializeToString,
            common_dot_common__pb2.Commands.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding:utf-8 -*-
# author：huawei


def cut(operation_name, threshold):
    return operation_name[0, threshold] if operation_name and len(
        operation_name) > threshold else operation_name


def is_empty(string):
    if string is None or string == "":
        return True
    return False



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: register/Register.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from python2sky.proto.common import common_pb2 as common_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='register/Register.proto',
  package='',
  syntax='proto3',
  serialized_options=b'\n-org.apache.skywalking.apm.network.register.v2P\001\252\002\032SkyWalking.NetworkProtocol',
  serialized_pb=b'\n\x17register/Register.proto\x1a\x13\x63ommon/common.proto\"&\n\x08Services\x12\x1a\n\x08services\x18\x01 \x03(\x0b\x32\x08.Service\"j\n\x07Service\x12\x13\n\x0bserviceName\x18\x01 \x01(\t\x12!\n\x04tags\x18\x03 \x03(\x0b\x32\x13.KeyStringValuePair\x12\'\n\nproperties\x18\x04 \x03(\x0b\x32\x13.KeyStringValuePair\"<\n\x16ServiceRegisterMapping\x12\"\n\x08services\x18\x01 \x03(\x0b\x32\x10.KeyIntValuePair\"7\n\x10ServiceInstances\x12#\n\tinstances\x18\x01 \x03(\x0b\x32\x10.ServiceInstance\"\x94\x01\n\x0fServiceInstance\x12\x11\n\tserviceId\x18\x01 \x01(\x05\x12\x14\n\x0cinstanceUUID\x18\x02 \x01(\t\x12\x0c\n\x04time\x18\x03 \x01(\x03\x12!\n\x04tags\x18\x04 \x03(\x0b\x32\x13.KeyStringValuePair\x12\'\n\nproperties\x18\x05 \x03(\x0b\x32\x13.KeyStringValuePair\"L\n\x1eServiceInstanceRegisterMapping\x12*\n\x10serviceInstances\x18\x01 \x03(\x0b\x32\x10.KeyIntValuePair\"!\n\x0cNetAddresses\x12\x11\n\taddresses\x18\x01 \x03(\t\"9\n\x11NetAddressMapping\x12$\n\naddressIds\x18\x01 \x03(\x0b\x32\x10.KeyIntValuePair\")\n\tEndpoints\x12\x1c\n\tendpoints\x18\x01 \x03(\x0b\x32\t.Endpoint\"\x9b\x01\n\x08\x45ndpoint\x12\x11\n\tserviceId\x18\x01 \x01(\x05\x12\x14\n\x0c\x65ndpointName\x18\x02 \x01(\t\x12!\n\x04tags\x18\x03 \x03(\x0b\x32\x13.KeyStringValuePair\x12\'\n\nproperties\x18\x04 \x03(\x0b\x32\x13.KeyStringValuePair\x12\x1a\n\x04\x66rom\x18\x05 \x01(\x0e\x32\x0c.DetectPoint\"<\n\x0f\x45ndpointMapping\x12)\n\x08\x65lements\x18\x01 \x03(\x0b\x32\x17.EndpointMappingElement\"q\n\x16\x45ndpointMappingElement\x12\x11\n\tserviceId\x18\x01 \x01(\x05\x12\x14\n\x0c\x65ndpointName\x18\x02 \x01(\t\x12\x12\n\nendpointId\x18\x03 \x01(\x05\x12\x1a\n\x04\x66rom\x18\x04 \x01(\x0e\x32\x0c.DetectPoint\"V\n ServiceAndNetworkAddressMappings\x12\x32\n\x08mappings\x18\x01 \x03(\x0b\x32 .ServiceAndNetworkAddressMapping\"\x81\x01\n\x1fServiceAndNetworkAddressMapping\x12\x11\n\tserviceId\x18\x01 \x01(\x05\x12\x19\n\x11serviceInstanceId\x18\x02 \x01(\x05\x12\x16\n\x0enetworkAddress\x18\x03 \x01(\t\x12\x18\n\x10networkAddressId\x18\x04 \x01(\x05\x32\xec\x02\n\x08Register\x12\x39\n\x11\x64oServiceRegister\x12\t.Services\x1a\x17.ServiceRegisterMapping\"\x00\x12Q\n\x19\x64oServiceInstanceRegister\x12\x11.ServiceInstances\x1a\x1f.ServiceInstanceRegisterMapping\"\x00\x12\x34\n\x12\x64oEndpointRegister\x12\n.Endpoints\x1a\x10.EndpointMapping\"\x00\x12?\n\x18\x64oNetworkAddressRegister\x12\r.NetAddresses\x1a\x12.NetAddressMapping\"\x00\x12[\n)doServiceAndNetworkAddressMappingRegister\x12!.ServiceAndNetworkAddressMappings\x1a\t.Commands\"\x00\x42N\n-org.apache.skywalking.apm.network.register.v2P\x01\xaa\x02\x1aSkyWalking.NetworkProtocolb\x06proto3'
  ,
  dependencies=[common_dot_common__pb2.DESCRIPTOR,])




_SERVICES = _descriptor.Descriptor(
  name='Services',
  full_name='Services',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='services', full_name='Services.services', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=48,
  serialized_end=86,
)


_SERVICE = _descriptor.Descriptor(
  name='Service',
  full_name='Service',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='serviceName', full_name='Service.serviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tags', full_name='Service.tags', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='properties', full_name='Service.properties', index=2,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=88,
  serialized_end=194,
)


_SERVICEREGISTERMAPPING = _descriptor.Descriptor(
  name='ServiceRegisterMapping',
  full_name='ServiceRegisterMapping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='services', full_name='ServiceRegisterMapping.services', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=196,
  serialized_end=256,
)


_SERVICEINSTANCES = _descriptor.Descriptor(
  name='ServiceInstances',
  full_name='ServiceInstances',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instances', full_name='ServiceInstances.instances', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=258,
  serialized_end=313,
)


_SERVICEINSTANCE = _descriptor.Descriptor(
  name='ServiceInstance',
  full_name='ServiceInstance',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='serviceId', full_name='ServiceInstance.serviceId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='instanceUUID', full_name='ServiceInstance.instanceUUID', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='time', full_name='ServiceInstance.time', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tags', full_name='ServiceInstance.tags', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='properties', full_name='ServiceInstance.properties', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=316,
  serialized_end=464,
)


_SERVICEINSTANCEREGISTERMAPPING = _descriptor.Descriptor(
  name='ServiceInstanceRegisterMapping',
  full_name='ServiceInstanceRegisterMapping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='serviceInstances', full_name='ServiceInstanceRegisterMapping.serviceInstances', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=466,
  serialized_end=542,
)


_NETADDRESSES = _descriptor.Descriptor(
  name='NetAddresses',
  full_name='NetAddresses',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='addresses', full_name='NetAddresses.addresses', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=544,
  serialized_end=577,
)


_NETADDRESSMAPPING = _descriptor.Descriptor(
  name='NetAddressMapping',
  full_name='NetAddressMapping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='addressIds', full_name='NetAddressMapping.addressIds', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=579,
  serialized_end=636,
)


_ENDPOINTS = _descriptor.Descriptor(
  name='Endpoints',
  full_name='Endpoints',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='endpoints', full_name='Endpoints.endpoints', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=638,
  serialized_end=679,
)


_ENDPOINT = _descriptor.Descriptor(
  name='Endpoint',
  full_name='Endpoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='serviceId', full_name='Endpoint.serviceId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='endpointName', full_name='Endpoint.endpointName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tags', full_name='Endpoint.tags', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='properties', full_name='Endpoint.properties', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='from', full_name='Endpoint.from', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=682,
  serialized_end=837,
)


_ENDPOINTMAPPING = _descriptor.Descriptor(
  name='EndpointMapping',
  full_name='EndpointMapping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='elements', full_name='EndpointMapping.elements', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=839,
  serialized_end=899,
)


_ENDPOINTMAPPINGELEMENT = _descriptor.Descriptor(
  name='EndpointMappingElement',
  full_name='EndpointMappingElement',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='serviceId', full_name='EndpointMappingElement.serviceId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='endpointName', full_name='EndpointMappingElement.endpointName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='endpointId', full_name='EndpointMappingElement.endpointId', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='from', full_name='EndpointMappingElement.from', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=901,
  serialized_end=1014,
)


_SERVICEANDNETWORKADDRESSMAPPINGS = _descriptor.Descriptor(
  name='ServiceAndNetworkAddressMappings',
  full_name='ServiceAndNetworkAddressMappings',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mappings', full_name='ServiceAndNetworkAddressMappings.mappings', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1016,
  serialized_end=1102,
)


_SERVICEANDNETWORKADDRESSMAPPING = _descriptor.Descriptor(
  name='ServiceAndNetworkAddressMapping',
  full_name='ServiceAndNetworkAddressMapping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='serviceId', full_name='ServiceAndNetworkAddressMapping.serviceId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='serviceInstanceId', full_name='ServiceAndNetworkAddressMapping.serviceInstanceId', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='networkAddress', full_name='ServiceAndNetworkAddressMapping.networkAddress', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='networkAddressId', full_name='ServiceAndNetworkAddressMapping.networkAddressId', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1105,
  serialized_end=1234,
)

_SERVICES.fields_by_name['services'].message_type = _SERVICE
_SERVICE.fields_by_name['tags'].message_type = common_dot_common__pb2._KEYSTRINGVALUEPAIR
_SERVICE.fields_by_name['properties'].message_type = common_dot_common__pb2._KEYSTRINGVALUEPAIR
_SERVICEREGISTERMAPPING.fields_by_name['services'].message_type = common_dot_common__pb2._KEYINTVALUEPAIR
_SERVICEINSTANCES.fields_by_name['instances'].message_type = _SERVICEINSTANCE
_SERVICEINSTANCE.fields_by_name['tags'].message_type = common_dot_common__pb2._KEYSTRINGVALUEPAIR
_SERVICEINSTANCE.fields_by_name['properties'].message_type = common_dot_common__pb2._KEYSTRINGVALUEPAIR
_SERVICEINSTANCEREGISTERMAPPING.fields_by_name['serviceInstances'].message_type = common_dot_common__pb2._KEYINTVALUEPAIR
_NETADDRESSMAPPING.fields_by_name['addressIds'].message_type = common_dot_common__pb2._KEYINTVALUEPAIR
_ENDPOINTS.fields_by_name['endpoints'].message_type = _ENDPOINT
_ENDPOINT.fields_by_name['tags'].message_type = common_dot_common__pb2._KEYSTRINGVALUEPAIR
_ENDPOINT.fields_by_name['properties'].message_type = common_dot_common__pb2._KEYSTRINGVALUEPAIR
_ENDPOINT.fields_by_name['from'].enum_type = common_dot_common__pb2._DETECTPOINT
_ENDPOINTMAPPING.fields_by_name['elements'].message_type = _ENDPOINTMAPPINGELEMENT
_ENDPOINTMAPPINGELEMENT.fields_by_name['from'].enum_type = common_dot_common__pb2._DETECTPOINT
_SERVICEANDNETWORKADDRESSMAPPINGS.fields_by_name['mappings'].message_type = _SERVICEANDNETWORKADDRESSMAPPING
DESCRIPTOR.message_types_by_name['Services'] = _SERVICES
DESCRIPTOR.message_types_by_name['Service'] = _SERVICE
DESCRIPTOR.message_types_by_name['ServiceRegisterMapping'] = _SERVICEREGISTERMAPPING
DESCRIPTOR.message_types_by_name['ServiceInstances'] = _SERVICEINSTANCES
DESCRIPTOR.message_types_by_name['ServiceInstance'] = _SERVICEINSTANCE
DESCRIPTOR.message_types_by_name['ServiceInstanceRegisterMapping'] = _SERVICEINSTANCEREGISTERMAPPING
DESCRIPTOR.message_types_by_name['NetAddresses'] = _NETADDRESSES
DESCRIPTOR.message_types_by_name['NetAddressMapping'] = _NETADDRESSMAPPING
DESCRIPTOR.message_types_by_name['Endpoints'] = _ENDPOINTS
DESCRIPTOR.message_types_by_name['Endpoint'] = _ENDPOINT
DESCRIPTOR.message_types_by_name['EndpointMapping'] = _ENDPOINTMAPPING
DESCRIPTOR.message_types_by_name['EndpointMappingElement'] = _ENDPOINTMAPPINGELEMENT
DESCRIPTOR.message_types_by_name['ServiceAndNetworkAddressMappings'] = _SERVICEANDNETWORKADDRESSMAPPINGS
DESCRIPTOR.message_types_by_name['ServiceAndNetworkAddressMapping'] = _SERVICEANDNETWORKADDRESSMAPPING
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Services = _reflection.GeneratedProtocolMessageType('Services', (_message.Message,), {
  'DESCRIPTOR' : _SERVICES,
  '__module__' : 'register.Register_pb2'
  # @@protoc_insertion_point(class_scope:Services)
  })
_sym_db.RegisterMessage(Services)

Service = _reflection.GeneratedProtocolMessageType('Service', (_message.Message,), {
  'DESCRIPTOR' : _SERVICE,
  '__module__' : 'register.Register_pb2'
  # @@protoc_insertion_point(class_scope:Service)
  })
_sym_db.RegisterMessage(Service)

ServiceRegisterMapping = _reflection.GeneratedProtocolMessageType('ServiceRegisterMapping', (_message.Message,), {
  'DESCRIPTOR' : _SERVICEREGISTERMAPPING,
  '__module__' : 'register.Register_pb2'
  # @@protoc_insertion_point(class_scope:ServiceRegisterMapping)
  })
_sym_db.RegisterMessage(ServiceRegisterMapping)

ServiceInstances = _reflection.GeneratedProtocolMessageType('ServiceInstances', (_message.Message,), {
  'DESCRIPTOR' : _SERVICEINSTANCES,
  '__module__' : 'register.Register_pb2'
  # @@protoc_insertion_point(class_scope:ServiceInstances)
  })
_sym_db.RegisterMessage(ServiceInstances)

ServiceInstance = _reflection.GeneratedProtocolMessageType('ServiceInstance', (_message.Message,), {
  'DESCRIPTOR' : _SERVICEINSTANCE,
  '__module__' : 'register.Register_pb2'
  # @@protoc_insertion_point(class_scope:ServiceInstance)
  })
_sym_db.RegisterMessage(ServiceInstance)

ServiceInstanceRegisterMapping = _reflection.GeneratedProtocolMessageType('ServiceInstanceRegisterMapping', (_message.Message,), {
  'DESCRIPTOR' : _SERVICEINSTANCEREGISTERMAPPING,
  '__module__' : 'register.Register_pb2'
  # @@protoc_insertion_point(class_scope:ServiceInstanceRegisterMapping)
  })
_sym_db.RegisterMessage(ServiceInstanceRegisterMapping)

NetAddresses = _reflection.GeneratedProtocolMessageType('NetAddresses', (_message.Message,), {
  'DESCRIPTOR' : _NETADDRESSES,
  '__module__' : 'register.Register_pb2'
  # @@protoc_insertion_point(class_scope:NetAddresses)
  })
_sym_db.RegisterMessage(NetAddresses)

NetAddressMapping = _reflection.GeneratedProtocolMessageType('NetAddressMapping', (_message.Message,), {
  'DESCRIPTOR' : _NETADDRESSMAPPING,
  '__module__' : 'register.Register_pb2'
  # @@protoc_insertion_point(class_scope:NetAddressMapping)
  })
_sym_db.RegisterMessage(NetAddressMapping)

Endpoints = _reflection.GeneratedProtocolMessageType('Endpoints', (_message.Message,), {
  'DESCRIPTOR' : _ENDPOINTS,
  '__module__' : 'register.Register_pb2'
  # @@protoc_insertion_point(class_scope:Endpoints)
  })
_sym_db.RegisterMessage(Endpoints)

Endpoint = _reflection.GeneratedProtocolMessageType('Endpoint', (_message.Message,), {
  'DESCRIPTOR' : _ENDPOINT,
  '__module__' : 'register.Register_pb2'
  # @@protoc_insertion_point(class_scope:Endpoint)
  })
_sym_db.RegisterMessage(Endpoint)

EndpointMapping = _reflection.GeneratedProtocolMessageType('EndpointMapping', (_message.Message,), {
  'DESCRIPTOR' : _ENDPOINTMAPPING,
  '__module__' : 'register.Register_pb2'
  # @@protoc_insertion_point(class_scope:EndpointMapping)
  })
_sym_db.RegisterMessage(EndpointMapping)

EndpointMappingElement = _reflection.GeneratedProtocolMessageType('EndpointMappingElement', (_message.Message,), {
  'DESCRIPTOR' : _ENDPOINTMAPPINGELEMENT,
  '__module__' : 'register.Register_pb2'
  # @@protoc_insertion_point(class_scope:EndpointMappingElement)
  })
_sym_db.RegisterMessage(EndpointMappingElement)

ServiceAndNetworkAddressMappings = _reflection.GeneratedProtocolMessageType('ServiceAndNetworkAddressMappings', (_message.Message,), {
  'DESCRIPTOR' : _SERVICEANDNETWORKADDRESSMAPPINGS,
  '__module__' : 'register.Register_pb2'
  # @@protoc_insertion_point(class_scope:ServiceAndNetworkAddressMappings)
  })
_sym_db.RegisterMessage(ServiceAndNetworkAddressMappings)

ServiceAndNetworkAddressMapping = _reflection.GeneratedProtocolMessageType('ServiceAndNetworkAddressMapping', (_message.Message,), {
  'DESCRIPTOR' : _SERVICEANDNETWORKADDRESSMAPPING,
  '__module__' : 'register.Register_pb2'
  # @@protoc_insertion_point(class_scope:ServiceAndNetworkAddressMapping)
  })
_sym_db.RegisterMessage(ServiceAndNetworkAddressMapping)


DESCRIPTOR._options = None

_REGISTER = _descriptor.ServiceDescriptor(
  name='Register',
  full_name='Register',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=1237,
  serialized_end=1601,
  methods=[
  _descriptor.MethodDescriptor(
    name='doServiceRegister',
    full_name='Register.doServiceRegister',
    index=0,
    containing_service=None,
    input_type=_SERVICES,
    output_type=_SERVICEREGISTERMAPPING,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='doServiceInstanceRegister',
    full_name='Register.doServiceInstanceRegister',
    index=1,
    containing_service=None,
    input_type=_SERVICEINSTANCES,
    output_type=_SERVICEINSTANCEREGISTERMAPPING,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='doEndpointRegister',
    full_name='Register.doEndpointRegister',
    index=2,
    containing_service=None,
    input_type=_ENDPOINTS,
    output_type=_ENDPOINTMAPPING,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='doNetworkAddressRegister',
    full_name='Register.doNetworkAddressRegister',
    index=3,
    containing_service=None,
    input_type=_NETADDRESSES,
    output_type=_NETADDRESSMAPPING,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='doServiceAndNetworkAddressMappingRegister',
    full_name='Register.doServiceAndNetworkAddressMappingRegister',
    index=4,
    containing_service=None,
    input_type=_SERVICEANDNETWORKADDRESSMAPPINGS,
    output_type=common_dot_common__pb2._COMMANDS,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_REGISTER)

DESCRIPTOR.services_by_name['Register'] = _REGISTER

# @@protoc_insertion_point(module_scope)



# -*- coding:utf-8 -*-
# author：huawei
from python2sky.proto.common.common_pb2 import KeyStringValuePair
from python2sky.proto.common.trace_common_pb2 import UniqueId


def build_key_value(key, value):
    pro = KeyStringValuePair()
    pro.key = key
    pro.value = str(value)
    return pro


def build_unique_id(trace_id):
    unique_id = UniqueId()
    unique_id.idParts.append(int(trace_id[0]))
    unique_id.idParts.append(int(trace_id[1]))
    unique_id.idParts.append(int(trace_id[2]))
    return unique_id


def null_value(id):
    if id == 0 or id is None:
        return True
    return False



# -*- coding:utf-8 -*-
# author：huawei
from python2sky import config
from python2sky.proto.common.trace_common_pb2 import UpstreamSegment
from python2sky.proto.language_agent_v2.trace_pb2 import SegmentObject
from python2sky.util.common import build_unique_id
from python2sky.util.date_util import current_milli_time
from python2sky.util.uuid_util import global_id_generator


class TraceSegment:
    def __init__(self):
        self.application_instance_id = config.SERVICE_INSTANCE_ID
        self.service_id = config.SERVICE_ID
        self.trace_segment_id = global_id_generator(self.application_instance_id)
        self.refs = []
        self.spans = []
        self.create_time = current_milli_time()
        self.id = global_id_generator(self.application_instance_id)
        self.trace_ids = [self.id]
        self.is_size_limited = False

    def ref(self, context_carrier):
        if context_carrier not in self.refs:
            self.refs.append(context_carrier)

    def related_global_traces(self, trace_id):
        if len(self.trace_ids) > 0 and self.id == self.trace_ids[0]:
            self.trace_ids.remove(self.id)

        if trace_id not in self.trace_ids:
            self.trace_ids.append(trace_id)

    def get_related_global_traces(self):
        return self.trace_ids

    def archive(self, span):
        span.end()
        self.spans.append(span)

    def transform(self):
        upstream_segment = UpstreamSegment()

        for trace_id in self.trace_ids:
            upstream_segment.globalTraceIds.append(build_unique_id(trace_id))

        segment_obj = SegmentObject()
        segment_obj.traceSegmentId.CopyFrom(build_unique_id(self.trace_segment_id))
        for span in self.spans:
            segment_obj.spans.append(span.transform())

        segment_obj.serviceId = self.service_id
        segment_obj.serviceInstanceId = self.application_instance_id
        segment_obj.isSizeLimited = self.is_size_limited

        upstream_segment.segment = segment_obj.SerializeToString()

        return upstream_segment



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from python2sky.proto.common import common_pb2 as common_dot_common__pb2
from python2sky.proto.register import InstancePing_pb2 as register_dot_InstancePing__pb2


class ServiceInstancePingStub(object):
    """Missing associated documentation comment in .proto file"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.doPing = channel.unary_unary(
                '/ServiceInstancePing/doPing',
                request_serializer=register_dot_InstancePing__pb2.ServiceInstancePingPkg.SerializeToString,
                response_deserializer=common_dot_common__pb2.Commands.FromString,
                )


class ServiceInstancePingServicer(object):
    """Missing associated documentation comment in .proto file"""

    def doPing(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ServiceInstancePingServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'doPing': grpc.unary_unary_rpc_method_handler(
                    servicer.doPing,
                    request_deserializer=register_dot_InstancePing__pb2.ServiceInstancePingPkg.FromString,
                    response_serializer=common_dot_common__pb2.Commands.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'ServiceInstancePing', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ServiceInstancePing(object):
    """Missing associated documentation comment in .proto file"""

    @staticmethod
    def doPing(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ServiceInstancePing/doPing',
            register_dot_InstancePing__pb2.ServiceInstancePingPkg.SerializeToString,
            common_dot_common__pb2.Commands.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding:utf-8 -*-
# author：huawei
import threading

import grpc

from python2sky import config
from python2sky.proto.register.InstancePing_pb2 import ServiceInstancePingPkg
from python2sky.proto.register.InstancePing_pb2_grpc import ServiceInstancePingStub
from python2sky.proto.register.Register_pb2 import Service, Services, ServiceInstance, ServiceInstances
from python2sky.proto.register.Register_pb2_grpc import RegisterStub
from python2sky.util.date_util import current_milli_time
from python2sky.util.os_util import get_os_name, get_all_ipv4, get_host_name, get_process_no
from python2sky.util.uuid_util import get_uuid
from python2sky.util.common import build_key_value
import logging

log = logging.getLogger(__name__)


class RegisterClient(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        self.__finished = threading.Event()
        self.service_name = config.SERVICE_NAME
        self.agent_version = config.AGENT_VERSION
        self.server = config.BACKEND_SERVICE
        self.service_id = None
        self.service_instance_id = None
        self.uuid = get_uuid()
        self.channel = None
        self.register_stub = None
        self.ping_stub = None
        self.connection()
        self.start()

    def run(self):
        while not self.__finished.is_set():
            try:
                if self.service_id is None:
                    service_register_mapping = self.register_service()
                    self.set_service_id(service_register_mapping)
                elif self.service_instance_id is None:
                    service_instance_register_mapping = self.register_instance(self.service_id, self.uuid)
                    self.set_service_instance_id(service_instance_register_mapping)
                else:
                    self.instance_ping(self.service_instance_id, self.uuid)

                self.__finished.wait(config.REGISTER_INTERVAL)
            except Exception as e:
                log.error(
                    "register execute fail will be Selected collector grpc service running, reconnect: %s ." % self.server)
                self.__finished.wait(config.REGISTER_INTERVAL)
                self.connection()

    def connection(self):
        try:

            self.channel = grpc.insecure_channel(self.server)
            self.register_stub = RegisterStub(self.channel)
            self.ping_stub = ServiceInstancePingStub(self.channel)
        except Exception as e:
            log.error("Create channel to %s fail." % self.server)

    def set_service_instance_id(self, service_instance_register_mapping):
        if service_instance_register_mapping and len(service_instance_register_mapping.serviceInstances) > 0:
            self.service_instance_id = service_instance_register_mapping.serviceInstances[0].value
            config.SERVICE_INSTANCE_ID = self.service_instance_id

    def set_service_id(self, service_register_mapping):
        if service_register_mapping and len(service_register_mapping.services) > 0:
            self.service_id = service_register_mapping.services[0].value
            config.SERVICE_ID = self.service_id

    def register_service(self):
        service = Service()
        service.serviceName = self.service_name
        services = Services()
        services.services.append(service)
        return self.register_stub.doServiceRegister(services)

    def register_instance(self, service_id, instance_uuid):
        service_instance = ServiceInstance()
        service_instance.serviceId = service_id
        service_instance.instanceUUID = instance_uuid
        service_instance.time = current_milli_time()

        service_instance.properties.append(build_key_value("os_name", get_os_name()))
        service_instance.properties.append(build_key_value("host_name", get_host_name()))
        ipv4s = get_all_ipv4()
        for v in ipv4s:
            service_instance.properties.append(build_key_value("ipv4", v))

        service_instance.properties.append(build_key_value("process_no", get_process_no()))
        service_instance.properties.append(build_key_value("language", "python"))

        service_instances = ServiceInstances()
        service_instances.instances.append(service_instance)
        return self.register_stub.doServiceInstanceRegister(service_instances)

    def instance_ping(self, instance_id, instance_uuid):
        ping = ServiceInstancePingPkg()
        ping.serviceInstanceId = instance_id
        ping.time = current_milli_time()
        ping.serviceInstanceUUID = instance_uuid
        config.SERVICE_ID = self.service_id
        config.SERVICE_INSTANCE_ID = self.service_instance_id
        return self.ping_stub.doPing(ping)


__register = RegisterClient()


def get_service_register():
    return __register


if __name__ == "__main__":
    register = RegisterClient()
    # register.start()
    # serviceRegisterMapping = register_service()
    # service_id = serviceRegisterMapping.services[0].value
    # uuid = get_uuid()
    # serviceInstanceRegisterMapping = register_instance(service_id, uuid)
    # print(serviceRegisterMapping)
    # print(serviceInstanceRegisterMapping.serviceInstances)
    # serviceInstanceRegisterMapping = register_instance(service_id, uuid)
    # serviceInstanceRegisterMapping = register_instance(service_id, uuid)
    # serviceInstanceRegisterMapping = register_instance(service_id, uuid)
    # print(serviceInstanceRegisterMapping.serviceInstances)



# -*- coding:utf-8 -*-
# author： huawei
import time
from datetime import datetime, timedelta


def format_value(obj):
    if obj and type(obj) == datetime:
        return obj.strftime("%Y-%m-%d %H:%M:%S")
    return obj


def parse_value(obj, format_str="%Y-%m-%d %H:%M:%S"):
    if obj and obj != "":
        return datetime.strptime(obj, format_str)
    return obj


def format_value_msec(obj):
    if obj and type(obj) == datetime:
        return obj.strftime("%Y-%m-%d %H:%M:%S.%f")
    return obj


def parse_value_msec(obj, format_str="%Y-%m-%d %H:%M:%S.%f"):
    if obj and obj != "":
        return datetime.strptime(obj, format_str)
    return obj


def timestamp_to_datetime(time_stamp):
    time_stamp = float(time_stamp / 1000)
    time_obj = time.localtime(time_stamp)
    return time.strftime("%Y-%m-%d %H:%M:%S", time_obj)


def datetime_to_timestamp(datetime_obj):
    """将本地(local) datetime 格式的时间 (含毫秒) 转为毫秒时间戳
    :param datetime_obj: {datetime}2016-02-25 20:21:04.242000
    :return: 13 位的毫秒时间戳 1456402864242
    """
    local_timestamp = int(time.mktime(datetime_obj.timetuple()) * 1000.0 + datetime_obj.microsecond / 1000.0)
    return local_timestamp


def exchange_to_time_bucket(obj):
    obj = obj.replace("-", "")
    obj = obj.replace(" ", "")
    obj = obj.replace(":", "")
    return int(obj)


def add_day(datetime_obj, days):
    return datetime_obj + timedelta(days=days)


def subtract_day(datetime_obj, days):
    return datetime_obj - timedelta(days=days)


def start_time_duration_to_second_time_bucket(date_str, step="SECOND"):
    second_time_bucket = 0
    if step == "MONTH":
        second_time_bucket = exchange_to_time_bucket(date_str) * 100 * 100 * 100 * 100
    if step == "DAY":
        second_time_bucket = exchange_to_time_bucket(date_str) * 100 * 100 * 100
    if step == "HOUR":
        second_time_bucket = exchange_to_time_bucket(date_str) * 100 * 100
    if step == "MINUTE":
        second_time_bucket = exchange_to_time_bucket(date_str) * 100
    if step == "SECOND":
        second_time_bucket = exchange_to_time_bucket(date_str)

    return second_time_bucket


def end_time_duration_to_second_time_bucket(date_str, step="SECOND"):
    second_time_bucket = 0
    if step == "MONTH":
        second_time_bucket = (((exchange_to_time_bucket(date_str) * 100 + 99) * 100 + 99) * 100 + 99) * 100 + 99
    if step == "DAY":
        second_time_bucket = ((exchange_to_time_bucket(date_str) * 100 + 99) * 100 + 99) * 100 + 99
    if step == "HOUR":
        second_time_bucket = (exchange_to_time_bucket(date_str) * 100 + 99) * 100 + 99
    if step == "MINUTE":
        second_time_bucket = exchange_to_time_bucket(date_str) * 100 + 99
    if step == "SECOND":
        second_time_bucket = exchange_to_time_bucket(date_str)

    return second_time_bucket


def current_milli_time():
    return int(round(time.time() * 1000))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: common/JVM.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from proto.common import common_pb2 as common_dot_common__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
  name='common/JVM.proto',
  package='',
  syntax='proto3',
  serialized_options=b'\n0org.apache.skywalking.apm.network.language.agentP\001\252\002\032SkyWalking.NetworkProtocol',
  serialized_pb=b'\n\x10\x63ommon/JVM.proto\x1a\x13\x63ommon/common.proto\"w\n\tJVMMetric\x12\x0c\n\x04time\x18\x01 \x01(\x03\x12\x11\n\x03\x63pu\x18\x02 \x01(\x0b\x32\x04.CPU\x12\x17\n\x06memory\x18\x03 \x03(\x0b\x32\x07.Memory\x12\x1f\n\nmemoryPool\x18\x04 \x03(\x0b\x32\x0b.MemoryPool\x12\x0f\n\x02gc\x18\x05 \x03(\x0b\x32\x03.GC\"T\n\x06Memory\x12\x0e\n\x06isHeap\x18\x01 \x01(\x08\x12\x0c\n\x04init\x18\x02 \x01(\x03\x12\x0b\n\x03max\x18\x03 \x01(\x03\x12\x0c\n\x04used\x18\x04 \x01(\x03\x12\x11\n\tcommitted\x18\x05 \x01(\x03\"`\n\nMemoryPool\x12\x17\n\x04type\x18\x01 \x01(\x0e\x32\t.PoolType\x12\x0c\n\x04init\x18\x02 \x01(\x03\x12\x0b\n\x03max\x18\x03 \x01(\x03\x12\x0c\n\x04used\x18\x04 \x01(\x03\x12\x10\n\x08\x63ommited\x18\x05 \x01(\x03\"<\n\x02GC\x12\x19\n\x06phrase\x18\x01 \x01(\x0e\x32\t.GCPhrase\x12\r\n\x05\x63ount\x18\x02 \x01(\x03\x12\x0c\n\x04time\x18\x03 \x01(\x03*\x80\x01\n\x08PoolType\x12\x14\n\x10\x43ODE_CACHE_USAGE\x10\x00\x12\x10\n\x0cNEWGEN_USAGE\x10\x01\x12\x10\n\x0cOLDGEN_USAGE\x10\x02\x12\x12\n\x0eSURVIVOR_USAGE\x10\x03\x12\x11\n\rPERMGEN_USAGE\x10\x04\x12\x13\n\x0fMETASPACE_USAGE\x10\x05*\x1c\n\x08GCPhrase\x12\x07\n\x03NEW\x10\x00\x12\x07\n\x03OLD\x10\x01\x42Q\n0org.apache.skywalking.apm.network.language.agentP\x01\xaa\x02\x1aSkyWalking.NetworkProtocolb\x06proto3'
  ,
  dependencies=[common_dot_common__pb2.DESCRIPTOR,])

_POOLTYPE = _descriptor.EnumDescriptor(
  name='PoolType',
  full_name='PoolType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CODE_CACHE_USAGE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NEWGEN_USAGE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OLDGEN_USAGE', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SURVIVOR_USAGE', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PERMGEN_USAGE', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='METASPACE_USAGE', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=409,
  serialized_end=537,
)
_sym_db.RegisterEnumDescriptor(_POOLTYPE)

PoolType = enum_type_wrapper.EnumTypeWrapper(_POOLTYPE)
_GCPHRASE = _descriptor.EnumDescriptor(
  name='GCPhrase',
  full_name='GCPhrase',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NEW', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OLD', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=539,
  serialized_end=567,
)
_sym_db.RegisterEnumDescriptor(_GCPHRASE)

GCPhrase = enum_type_wrapper.EnumTypeWrapper(_GCPHRASE)
CODE_CACHE_USAGE = 0
NEWGEN_USAGE = 1
OLDGEN_USAGE = 2
SURVIVOR_USAGE = 3
PERMGEN_USAGE = 4
METASPACE_USAGE = 5
NEW = 0
OLD = 1



_JVMMETRIC = _descriptor.Descriptor(
  name='JVMMetric',
  full_name='JVMMetric',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='time', full_name='JVMMetric.time', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cpu', full_name='JVMMetric.cpu', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='memory', full_name='JVMMetric.memory', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='memoryPool', full_name='JVMMetric.memoryPool', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gc', full_name='JVMMetric.gc', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=41,
  serialized_end=160,
)


_MEMORY = _descriptor.Descriptor(
  name='Memory',
  full_name='Memory',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='isHeap', full_name='Memory.isHeap', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='init', full_name='Memory.init', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max', full_name='Memory.max', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='used', full_name='Memory.used', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='committed', full_name='Memory.committed', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=162,
  serialized_end=246,
)


_MEMORYPOOL = _descriptor.Descriptor(
  name='MemoryPool',
  full_name='MemoryPool',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='MemoryPool.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='init', full_name='MemoryPool.init', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max', full_name='MemoryPool.max', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='used', full_name='MemoryPool.used', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='commited', full_name='MemoryPool.commited', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=248,
  serialized_end=344,
)


_GC = _descriptor.Descriptor(
  name='GC',
  full_name='GC',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='phrase', full_name='GC.phrase', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='count', full_name='GC.count', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='time', full_name='GC.time', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=346,
  serialized_end=406,
)

_JVMMETRIC.fields_by_name['cpu'].message_type = common_dot_common__pb2._CPU
_JVMMETRIC.fields_by_name['memory'].message_type = _MEMORY
_JVMMETRIC.fields_by_name['memoryPool'].message_type = _MEMORYPOOL
_JVMMETRIC.fields_by_name['gc'].message_type = _GC
_MEMORYPOOL.fields_by_name['type'].enum_type = _POOLTYPE
_GC.fields_by_name['phrase'].enum_type = _GCPHRASE
DESCRIPTOR.message_types_by_name['JVMMetric'] = _JVMMETRIC
DESCRIPTOR.message_types_by_name['Memory'] = _MEMORY
DESCRIPTOR.message_types_by_name['MemoryPool'] = _MEMORYPOOL
DESCRIPTOR.message_types_by_name['GC'] = _GC
DESCRIPTOR.enum_types_by_name['PoolType'] = _POOLTYPE
DESCRIPTOR.enum_types_by_name['GCPhrase'] = _GCPHRASE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

JVMMetric = _reflection.GeneratedProtocolMessageType('JVMMetric', (_message.Message,), {
  'DESCRIPTOR' : _JVMMETRIC,
  '__module__' : 'common.JVM_pb2'
  # @@protoc_insertion_point(class_scope:JVMMetric)
  })
_sym_db.RegisterMessage(JVMMetric)

Memory = _reflection.GeneratedProtocolMessageType('Memory', (_message.Message,), {
  'DESCRIPTOR' : _MEMORY,
  '__module__' : 'common.JVM_pb2'
  # @@protoc_insertion_point(class_scope:Memory)
  })
_sym_db.RegisterMessage(Memory)

MemoryPool = _reflection.GeneratedProtocolMessageType('MemoryPool', (_message.Message,), {
  'DESCRIPTOR' : _MEMORYPOOL,
  '__module__' : 'common.JVM_pb2'
  # @@protoc_insertion_point(class_scope:MemoryPool)
  })
_sym_db.RegisterMessage(MemoryPool)

GC = _reflection.GeneratedProtocolMessageType('GC', (_message.Message,), {
  'DESCRIPTOR' : _GC,
  '__module__' : 'common.JVM_pb2'
  # @@protoc_insertion_point(class_scope:GC)
  })
_sym_db.RegisterMessage(GC)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding:utf-8 -*-
# author：huawei
import threading


# copy ....

class CountDownLatch(object):
    def __init__(self, count=1):
        self.count = count
        self.lock = threading.Condition()

    def count_down(self):
        self.lock.acquire()
        self.count -= 1
        if self.count <= 0:
            self.lock.notifyAll()
        self.lock.release()

    def wait(self):
        self.lock.acquire()
        while self.count > 0:
            self.lock.wait()
        self.lock.release()



# -*- coding:utf-8 -*-
# author：huawei


URL = "url"

STATUS_CODE = "status_code"

DB_TYPE = "db.type"

DB_INSTANCE = "db.instance"

DB_STATEMENT = "db.statement"

DB_BIND_VARIABLES = "db.bind_vars"

MQ_QUEUE = "mq.queue"

MQ_BROKER = "mq.broker"

MQ_TOPIC = "mq.topic"

METHOD = "http.method"

UNKNOWN = 0
DATABASE = 1
RPC_FRAMEWORK = 2
HTTP = 3
MQ = 4
CACHE = 5


class Component:
    def __init__(self, id, name):
        self.id = id
        self.name = name


FLASK = Component(6001, "Flask")
REQUESTS = Component(6002, "requests")


def set_component(span, component):
    span.component_id = component.name
    span.component_id = component.id


def set_layer_db(span):
    span.layer = DATABASE


def set_layer_rpc_framework(span):
    span.layer = RPC_FRAMEWORK


def set_layer_http(span):
    span.layer = HTTP


def set_layer_mq(span):
    span.layer = MQ


def set_layer_cache(span):
    span.layer = CACHE


def set_tag_url(span, v):
    span.tag(URL, v)


def set_tag_method(span, v):
    span.tag(METHOD, v)


def set_tag_status_code(span, v):
    span.tag(STATUS_CODE, v)


def set_tag_db_type(span, v):
    span.tag(DB_TYPE, v)


def set_tag_db_instance(span, v):
    span.tag(DB_INSTANCE, v)


def set_tag_db_statement(span, v):
    span.tag(DB_STATEMENT, v)


def set_tag_db_bind_variables(span, v):
    span.tag(DB_BIND_VARIABLES, v)


def set_tag_mq_queue(span, v):
    span.tag(MQ_QUEUE, v)


def set_tag_mq_broker(span, v):
    span.tag(MQ_BROKER, v)


def set_tag_mq_topic(span, v):
    span.tag(MQ_TOPIC, v)



# -*- coding:utf-8 -*-
# author：huawei
import logging
import threading
import time
from queue import Queue

import grpc

from python2sky import config
from python2sky.context.trace_context import ListenerManager
from python2sky.proto.language_agent_v2.trace_pb2_grpc import TraceSegmentReportServiceStub

log = logging.getLogger(__name__)


class TraceSegmentClient(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        self.__finished = threading.Event()
        self.q = Queue()
        self.server = config.BACKEND_SERVICE
        self.service_id = config.SERVICE_ID
        self.service_instance_id = config.SERVICE_INSTANCE_ID
        self.channel = None
        self.register_stub = None
        self.trace_segment_stub = None
        ListenerManager.add(self)
        self.state = ""
        self.connection()
        self.start()

    def run(self):
        while not self.__finished.is_set():
            try:
                if not self.connected():
                    self.connection()

                def iter_data():
                    trace_segment = self.q.get()
                    yield trace_segment.transform()
                    self.q.task_done()

                self.trace_segment_stub.collect(iter_data())

            except Exception as e:
                log.error("Transform and send UpstreamSegment to collector fail.%s." % self.server)
                self.__finished.wait(config.REGISTER_INTERVAL)
                self.channel.close()
                self.connection()

    def connection(self):
        try:
            self.channel = grpc.insecure_channel(self.server)
            self.trace_segment_stub = TraceSegmentReportServiceStub(self.channel)
            self.state = grpc.ChannelConnectivity.READY
        except Exception as e:
            self.channel.close()
            self.state = grpc.ChannelConnectivity.SHUTDOWN
            log.error("Create channel to %s fail." % self.server)

    def after_finished(self, trace_segment):
        self.q.put(trace_segment)

    def connected(self):
        return self.state == grpc.ChannelConnectivity.READY


__trace_segment_client = TraceSegmentClient()


def get_trace_segment_client():
    return __trace_segment_client


if __name__ == "__main__":
    get_trace_segment_client()



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# -*- coding:utf-8 -*-

# author：huawei
# from https://github.com/apache/skywalking-python/blob/master/skywalking/plugins/sw_urllib_request/__init__.py
import logging
import traceback
from urllib.request import Request, OpenerDirector

from python2sky.config import SKYWALKING_HERADER_V2
from python2sky.context.common import set_layer_http, set_component, set_tag_status_code, set_tag_url, set_tag_method
from python2sky.context.context_carrier import ContextCarrier
from python2sky.context.context_manager import ContextManager

logger = logging.getLogger(__name__)


def urllib_install():
    # noinspection PyBroadException
    try:
        from urllib.request import OpenerDirector
        from urllib.error import HTTPError

        _open = OpenerDirector.open

        def _sw_open(this: OpenerDirector, fullurl, data, timeout):
            if isinstance(fullurl, str):
                fullurl = Request(fullurl, data)

            context_carrier = ContextCarrier()

            exit_span = ContextManager.create_inject_exit_span(fullurl.selector or '/', fullurl.host, context_carrier)
            set_layer_http(exit_span)
            set_component(exit_span, "urllib")
            fullurl.add_header(SKYWALKING_HERADER_V2, context_carrier.serialize())
            try:
                res = _open(this, fullurl, data, timeout)
                set_tag_status_code(exit_span, res.code)
                set_tag_url(exit_span, fullurl.full_url)
                set_tag_method(exit_span, fullurl.get_method())

                if res.code >= 400:
                    exit_span.error_occurred = True
            except HTTPError as e:
                exit_span.log(e)
                raise e
            finally:
                ContextManager.stop_span(exit_span)
            return res

        OpenerDirector.open = _sw_open
    except Exception:
        logger.warning('failed to install plugin %s', __name__)
        traceback.print_exc()



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# -*- coding:utf-8 -*-

# author：huawei
# from https://github.com/apache/skywalking-python/blob/master/skywalking/plugins/sw_urllib_request/__init__.py
import logging
import traceback

from python2sky.config import SKYWALKING_HERADER_V2
from python2sky.context.common import set_layer_http, set_component, set_tag_status_code, set_tag_url, set_tag_method, \
    REQUESTS
from python2sky.context.context_carrier import ContextCarrier
from python2sky.context.context_manager import ContextManager

logger = logging.getLogger(__name__)


def requests_install():
    # noinspection PyBroadException
    try:
        from requests import Session

        _request = Session.request

        def _sw_request(this: Session, method, url,
                     params=None, data=None, headers=None, cookies=None, files=None,
                     auth=None, timeout=None, allow_redirects=True, proxies=None,
                     hooks=None, stream=None, verify=None, cert=None, json=None):

            context_carrier = ContextCarrier()
            from urllib.parse import urlparse
            res = urlparse(url)

            exit_span = ContextManager.create_inject_exit_span(res.path, res.netloc, context_carrier)
            set_layer_http(exit_span)
            set_component(exit_span, REQUESTS)

            if headers is None:
                headers = {SKYWALKING_HERADER_V2: context_carrier.serialize()}
            else:
                headers[SKYWALKING_HERADER_V2] = context_carrier.serialize()

            try:
                res = _request(this, method, url, params, data, headers, cookies, files, auth, timeout, allow_redirects,
                               proxies,
                               hooks, stream, verify, cert, json)
                set_tag_status_code(exit_span, res.status_code)
                set_tag_url(exit_span, url)
                set_tag_method(exit_span, method)

                if res.status_code >= 400:
                    exit_span.error_occurred = True
            except BaseException as e:
                exit_span.log(e)
                raise e
            finally:
                ContextManager.stop_span(exit_span)
            return res

        Session.request = _sw_request
    except Exception:
        logger.warning('failed to install plugin %s', __name__)
        traceback.print_exc()



# -*- coding:utf-8 -*-
# author：huawei
import functools

from python2sky.config import SKYWALKING_HERADER_V2
from python2sky.context.common import set_tag_url, set_tag_method, set_layer_http, set_component, FLASK
from python2sky.context.context_carrier import ContextCarrier
from python2sky.context.context_manager import ContextManager
from python2sky.exception.exceptions import SkywalkingException


def trace_flask_request(f):
    @functools.wraps(f)
    def decorated_function(*args, **kwargs):
        try:
            import flask
        except ImportError:
            return f(*args, **kwargs)
        req = flask.request
        sw6 = req.headers.get(SKYWALKING_HERADER_V2)
        context_carrier = None
        if sw6 and sw6 != "":
            context_carrier = ContextCarrier()
            context_carrier.deserialize(sw6)
        entry_span = ContextManager.create_entry_span(req.path, context_carrier)
        set_tag_url(entry_span, req.url)
        set_tag_method(entry_span, req.method)
        set_layer_http(entry_span)
        set_component(entry_span, FLASK)
        try:
            return f(*args, **kwargs)
        except BaseException as ex:
            entry_span.log(ex)
        finally:
            ContextManager.stop_span(entry_span)

    return decorated_function


def trace_request_started(sender, **extra):
    try:
        import flask
    except ImportError:
        return
    req = flask.request
    sw6 = req.headers.get(SKYWALKING_HERADER_V2)
    context_carrier = None
    if sw6 and sw6 != "":
        context_carrier = ContextCarrier()
        context_carrier.deserialize(sw6)
    entry_span = ContextManager.create_entry_span(req.path, context_carrier)
    set_tag_url(entry_span, req.url)
    set_tag_method(entry_span, req.method)
    set_layer_http(entry_span)
    set_component(entry_span, FLASK)


def trace_request_exception(exception=None):
    try:
        import flask
    except ImportError:
        pass
    try:
        span = ContextManager.active_span()
        if exception is not None:
            span.log(exception)
        ContextManager.stop_span(span)
    except SkywalkingException as ex:
        pass


def flask_install(app):
    try:
        import flask
    except ImportError:
        return
    from flask import request_started
    request_started.connect(trace_request_started, app)
    app.teardown_request(trace_request_exception)



# -*- coding:utf-8 -*-
# author：huawei

import base64
UTF8 = "utf-8"


def decode(text):
    return str(base64.b64decode(text), UTF8)


def encode(text):
    return str(base64.b64encode(text.encode(UTF8)), UTF8)


if __name__ == "__main__":
    d = "test"
    d = encode(d)
    print(d)
    d = decode(d)
    print(d)


# -*- coding:utf-8 -*-
# author：huawei
import sys
from functools import wraps

from python2sky.context.context_manager import ContextManager


def trace(operation_name=None):
    def wrapper(f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            local_span = ContextManager.create_local_span(operation_name or "/"+f.__name__)
            try:
                return f(*args, **kwargs)
            except BaseException as ex:
                local_span.log(ex)
                raise ex
            finally:
                ContextManager.stop_span(local_span)

        return decorated_function
    return wrapper



# -*- coding:utf-8 -*-
# author：huawei
import logging
import threading

from python2sky.context.IgnoredTracerContext import IgnoredTracerContext
from python2sky import config
from python2sky.config import OPERATION_NAME_THRESHOLD
from python2sky.context.trace_context import TracingContext
from python2sky.exception.exceptions import SkywalkingException
from python2sky.util import string_util
from python2sky.util.common import null_value

log = logging.Logger(__name__)


class ContextManager:
    CONTEXT = threading.local()

    @classmethod
    def set_tracing_context(cls, tracing_context):
        ContextManager.CONTEXT.trace_context = tracing_context

    @classmethod
    def set_runtime_context(cls, runtime_context):
        ContextManager.CONTEXT.runtime = runtime_context

    @classmethod
    def get_or_create(cls, operation_name, force_sampling):
        if not hasattr(ContextManager.CONTEXT, "trace_context"):
            if not operation_name:
                log.debug("No operation name, ignore this trace.")
                tracing_context = IgnoredTracerContext()
            else:
                if not null_value(config.SERVICE_ID) and not null_value(config.SERVICE_INSTANCE_ID):
                    tracing_context = ContextManager.create_trace_context(operation_name, force_sampling)
                else:
                    log.info("service not register skywalking server!")
                    tracing_context = IgnoredTracerContext()
        else:
            tracing_context = ContextManager.CONTEXT.trace_context
        ContextManager.CONTEXT.trace_context = tracing_context

        return tracing_context

    @classmethod
    def get_global_trace_id(cls):
        tracing_context = ContextManager.get_tracing_context()
        if tracing_context:
            return tracing_context.get_readable_global_trace_id()
        else:
            return "N/A"

    @classmethod
    def create_entry_span(cls, operation_name, context_carrier):
        operation_name = string_util.cut(operation_name, OPERATION_NAME_THRESHOLD)
        tracing_context = ContextManager.get_or_create(operation_name, True)
        entry_span = tracing_context.create_entry_span(operation_name)
        if context_carrier:
            tracing_context.extract(context_carrier)

        return entry_span

    @classmethod
    def create_local_span(cls, operation_name):
        operation_name = string_util.cut(operation_name, OPERATION_NAME_THRESHOLD)
        tracing_context = ContextManager.get_or_create(operation_name, True)
        local_span = tracing_context.create_local_span(operation_name)
        return local_span

    @classmethod
    def create_inject_exit_span(cls, operation_name, remote_peer, context_carrier):
        if not context_carrier:
            raise SkywalkingException("ContextCarrier can't be null.")
        operation_name = string_util.cut(operation_name, OPERATION_NAME_THRESHOLD)
        tracing_context = ContextManager.get_or_create(operation_name, True)
        exit_span = tracing_context.create_exit_span(operation_name, remote_peer)
        tracing_context.inject(context_carrier)
        return exit_span

    @classmethod
    def create_exit_span(cls, operation_name, remote_peer):
        operation_name = string_util.cut(operation_name, OPERATION_NAME_THRESHOLD)
        tracing_context = ContextManager.get_or_create(operation_name, True)
        exit_span = tracing_context.create_exit_span(operation_name, remote_peer)
        return exit_span

    @classmethod
    def inject(cls, context_carrier):
        ContextManager.get_tracing_context().inject(context_carrier)

    @classmethod
    def extract(cls, context_carrier):
        if not context_carrier:
            raise SkywalkingException("ContextCarrier can't be null.")
        ContextManager.get_tracing_context().extract(context_carrier)

    @classmethod
    def capture(cls):
        return ContextManager.get_tracing_context().capture()

    @classmethod
    def continued(cls, snapshot):
        if not snapshot:
            raise SkywalkingException("ContextSnapshot can't be null.")
        ContextManager.get_tracing_context().continued(snapshot)

    @classmethod
    def await_finish_async(cls, span):
        pass

    @classmethod
    def active_span(cls):
        return ContextManager.get_tracing_context().active_span()

    @classmethod
    def stop_active_span(cls):
        tracing_context = ContextManager.get_tracing_context()
        ContextManager.__stop_span(tracing_context.active_span(), tracing_context)

    @classmethod
    def stop_span(cls, span):
        ContextManager.__stop_span(span, ContextManager.get_tracing_context())

    @classmethod
    def __stop_span(cls, span, tracing_context):
        if tracing_context.stop_span(span):
            ContextManager.CONTEXT.__dict__.clear()

    @classmethod
    def get_tracing_context(cls):
        if hasattr(ContextManager.CONTEXT, "trace_context"):
            return ContextManager.CONTEXT.trace_context
        return None

    @classmethod
    def get_run_time_context(cls):
        if hasattr(ContextManager.CONTEXT, "runtime"):
            return ContextManager.CONTEXT.runtime
        return None

    @classmethod
    def create_trace_context(cls, operation_name, force_sampling):
        suffix_idx = operation_name.rfind(".")
        if suffix_idx > -1 and operation_name[operation_name.rfind(".")+1:] in config.IGNORE_SUFFIX:
            context = IgnoredTracerContext()
        else:
            if force_sampling:
                context = TracingContext()
            else:
                context = IgnoredTracerContext()

        return context



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: common/trace-common.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='common/trace-common.proto',
  package='',
  syntax='proto3',
  serialized_options=b'\n0org.apache.skywalking.apm.network.language.agentP\001\252\002\032SkyWalking.NetworkProtocol',
  serialized_pb=b'\n\x19\x63ommon/trace-common.proto\"E\n\x0fUpstreamSegment\x12!\n\x0eglobalTraceIds\x18\x01 \x03(\x0b\x32\t.UniqueId\x12\x0f\n\x07segment\x18\x02 \x01(\x0c\"\x1b\n\x08UniqueId\x12\x0f\n\x07idParts\x18\x01 \x03(\x03**\n\x08SpanType\x12\t\n\x05\x45ntry\x10\x00\x12\x08\n\x04\x45xit\x10\x01\x12\t\n\x05Local\x10\x02*,\n\x07RefType\x12\x10\n\x0c\x43rossProcess\x10\x00\x12\x0f\n\x0b\x43rossThread\x10\x01*U\n\tSpanLayer\x12\x0b\n\x07Unknown\x10\x00\x12\x0c\n\x08\x44\x61tabase\x10\x01\x12\x10\n\x0cRPCFramework\x10\x02\x12\x08\n\x04Http\x10\x03\x12\x06\n\x02MQ\x10\x04\x12\t\n\x05\x43\x61\x63he\x10\x05\x42Q\n0org.apache.skywalking.apm.network.language.agentP\x01\xaa\x02\x1aSkyWalking.NetworkProtocolb\x06proto3'
)

_SPANTYPE = _descriptor.EnumDescriptor(
  name='SpanType',
  full_name='SpanType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Entry', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Exit', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Local', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=129,
  serialized_end=171,
)
_sym_db.RegisterEnumDescriptor(_SPANTYPE)

SpanType = enum_type_wrapper.EnumTypeWrapper(_SPANTYPE)
_REFTYPE = _descriptor.EnumDescriptor(
  name='RefType',
  full_name='RefType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CrossProcess', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CrossThread', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=173,
  serialized_end=217,
)
_sym_db.RegisterEnumDescriptor(_REFTYPE)

RefType = enum_type_wrapper.EnumTypeWrapper(_REFTYPE)
_SPANLAYER = _descriptor.EnumDescriptor(
  name='SpanLayer',
  full_name='SpanLayer',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Unknown', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Database', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RPCFramework', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Http', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MQ', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Cache', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=219,
  serialized_end=304,
)
_sym_db.RegisterEnumDescriptor(_SPANLAYER)

SpanLayer = enum_type_wrapper.EnumTypeWrapper(_SPANLAYER)
Entry = 0
Exit = 1
Local = 2
CrossProcess = 0
CrossThread = 1
Unknown = 0
Database = 1
RPCFramework = 2
Http = 3
MQ = 4
Cache = 5



_UPSTREAMSEGMENT = _descriptor.Descriptor(
  name='UpstreamSegment',
  full_name='UpstreamSegment',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='globalTraceIds', full_name='UpstreamSegment.globalTraceIds', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='segment', full_name='UpstreamSegment.segment', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=29,
  serialized_end=98,
)


_UNIQUEID = _descriptor.Descriptor(
  name='UniqueId',
  full_name='UniqueId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='idParts', full_name='UniqueId.idParts', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=100,
  serialized_end=127,
)

_UPSTREAMSEGMENT.fields_by_name['globalTraceIds'].message_type = _UNIQUEID
DESCRIPTOR.message_types_by_name['UpstreamSegment'] = _UPSTREAMSEGMENT
DESCRIPTOR.message_types_by_name['UniqueId'] = _UNIQUEID
DESCRIPTOR.enum_types_by_name['SpanType'] = _SPANTYPE
DESCRIPTOR.enum_types_by_name['RefType'] = _REFTYPE
DESCRIPTOR.enum_types_by_name['SpanLayer'] = _SPANLAYER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

UpstreamSegment = _reflection.GeneratedProtocolMessageType('UpstreamSegment', (_message.Message,), {
  'DESCRIPTOR' : _UPSTREAMSEGMENT,
  '__module__' : 'common.trace_common_pb2'
  # @@protoc_insertion_point(class_scope:UpstreamSegment)
  })
_sym_db.RegisterMessage(UpstreamSegment)

UniqueId = _reflection.GeneratedProtocolMessageType('UniqueId', (_message.Message,), {
  'DESCRIPTOR' : _UNIQUEID,
  '__module__' : 'common.trace_common_pb2'
  # @@protoc_insertion_point(class_scope:UniqueId)
  })
_sym_db.RegisterMessage(UniqueId)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding:utf-8 -*-
# author：huawei


from setuptools import setup, find_packages

packages = ['python2sky']

install_requires = ["grpcio", "grpcio-tools"]

test_requirements = [
    'pytest-httpbin==0.0.7',
    'pytest-cov',
    'pytest-mock',
    'pytest-xdist',
    'PySocks>=1.5.6, !=1.5.7',
    'pytest>=3'
]

setup(
    name="python2sky",
    version="0.2.2",
    keywords=("skywalking", "agent", "skywalking aget", "apm", "trace", "distributed trace"),
    description="skywalking agent for python",
    long_description="apm agent",
    license="apache Licence",

    url="https://github.com/alonelaval/python2sky",
    author="huawei",
    author_email="120huawei@163.com",

    include_package_data=True,
    platforms="any",
    packages=find_packages(exclude=['tests', 'test.*']),
    install_requires=install_requires

)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from python2sky.proto.common import common_pb2 as common_dot_common__pb2
from python2sky.proto.common import trace_common_pb2 as common_dot_trace__common__pb2


class TraceSegmentReportServiceStub(object):
    """Missing associated documentation comment in .proto file"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.collect = channel.stream_unary(
                '/TraceSegmentReportService/collect',
                request_serializer=common_dot_trace__common__pb2.UpstreamSegment.SerializeToString,
                response_deserializer=common_dot_common__pb2.Commands.FromString,
                )


class TraceSegmentReportServiceServicer(object):
    """Missing associated documentation comment in .proto file"""

    def collect(self, request_iterator, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TraceSegmentReportServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'collect': grpc.stream_unary_rpc_method_handler(
                    servicer.collect,
                    request_deserializer=common_dot_trace__common__pb2.UpstreamSegment.FromString,
                    response_serializer=common_dot_common__pb2.Commands.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'TraceSegmentReportService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class TraceSegmentReportService(object):
    """Missing associated documentation comment in .proto file"""

    @staticmethod
    def collect(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(request_iterator, target, '/TraceSegmentReportService/collect',
            common_dot_trace__common__pb2.UpstreamSegment.SerializeToString,
            common_dot_common__pb2.Commands.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)
